# functions used by RFC-1213 MIB test modules

myport=$SNMP_TRANSPORT_SPEC:$SNMP_TEST_DEST$SNMP_SNMPD_PORT

noauth=""  # no - use Auth+Priv . yes - no auth, no priv

if [ "x$noauth" = xyes ] ; then
   TEST_AUTHPRIV_PARMS="-l noAuthnoPriv"
else
   TEST_AUTHPRIV_PARMS="-l authNoPriv -a MD5 -A testpass"
fi

check_skip_arp_tests ()
{
	#
	# skip all tests relying on a filed arp table. e.g. on s390 boxes this
	# does not have to be the case
	#
	TABLE_ENTRIES="at.atTable ip.ipNetToMediaTable"
	ARP_COUNT=`/sbin/arp | grep -v incomplete | wc -l`
	for entry in $TABLE_ENTRIES ; do
		if [ "x$1" == "x$entry" -a $ARP_COUNT == 0 ] ; then
			echo "skipping $1, because the arp table is empty." >&2
			return 0
		fi
	done
	return 1
}

config()
{
	rm -f $SNMP_CONFIG_FILE
	CONFIGAGENT rwcommunity test
	STARTAGENT
}

configv3()
{
	rm -f $SNMP_CONFIG_FILE
	CONFIGAGENT rwcommunity test
	CONFIGAGENT rwuser testrwuser noauth
	CONFIGAGENT createUser testrwuser MD5 testpass
	STARTAGENT
}

get_snmp_variable()
{
	test_start "Access $2.0 by SNMPv$1..."
	CAPTURE "snmpget -v $1 -c test $myport $2.0"
	if [ $? != 0 ];then
		test_finish FAIL
	else
		test_finish PASS
	fi

}

get_snmpv3_variable()
{
	test_start "Access $2.0 by SNMPv3..."
	CAPTURE "snmpget -v 3 -u testrwuser $TEST_AUTHPRIV_PARMS $myport $2.0"
	if [ $? != 0 ];then
		test_finish FAIL
	else
		test_finish PASS
	fi

}

get_snmp_table()
{
	test_start "Access table $2 by SNMPv$1..."
	check_skip_arp_tests 
	if check_skip_arp_tests "$2" ; then
		test_finish SKIPPED
	else
		CAPTURE "snmpgetnext -Of -v $1 -c test $myport $2"
		CHECKFILE '' "\.$2\."
		if [ "$snmp_last_test_result" = 0 ] ; then
			test_finish FAIL
		else
			test_finish PASS
		fi
	fi

}

get_snmpv3_table()
{
	test_start "Access table $2 by SNMPv3..."
	if check_skip_arp_tests  "$2" ; then
		test_finish SKIPPED
	else
		CAPTURE "snmpgetnext -Of -v 3 -u testrwuser $TEST_AUTHPRIV_PARMS $myport $2"
		CHECKFILE '' "\.$2\."
		if [ "$snmp_last_test_result" = 0 ] ; then
			test_finish FAIL
		else
			test_finish PASS
		fi
	fi

}

